/* AUTO-GENERATED HEADER FILE.  DO NOT EDIT. */
#define PW_TUNNEL_TYPE 64
#define PW_TUNNEL_MEDIUM_TYPE 65
#define PW_TUNNEL_CLIENT_ENDPOINT 66
#define PW_TUNNEL_SERVER_ENDPOINT 67
#define PW_TUNNEL_PASSWORD 69
#define PW_TUNNEL_PRIVATE_GROUP_ID 81
#define PW_TUNNEL_ASSIGNMENT_ID 82
#define PW_TUNNEL_PREFERENCE 83
#define PW_TUNNEL_CLIENT_AUTH_ID 90
#define PW_TUNNEL_SERVER_AUTH_ID 91
