/* AUTO-GENERATED HEADER FILE.  DO NOT EDIT. */
#define PW_FRAMED_IPV6_ADDRESS 168
#define PW_DNS_SERVER_IPV6_ADDRESS 169
#define PW_ROUTE_IPV6_INFORMATION 170
#define PW_DELEGATED_IPV6_PREFIX_POOL 171
#define PW_STATEFUL_IPV6_ADDRESS_POOL 172
