/* AUTO-GENERATED HEADER FILE.  DO NOT EDIT. */
#define PW_ACCT_INPUT_GIGAWORDS 52
#define PW_ACCT_OUTPUT_GIGAWORDS 53
#define PW_EVENT_TIMESTAMP 55
#define PW_ARAP_PASSWORD 70
#define PW_ARAP_FEATURES 71
#define PW_ARAP_ZONE_ACCESS 72
#define PW_ARAP_SECURITY 73
#define PW_ARAP_SECURITY_DATA 74
#define PW_PASSWORD_RETRY 75
#define PW_PROMPT 76
#define PW_CONNECT_INFO 77
#define PW_CONFIGURATION_TOKEN 78
#define PW_EAP_MESSAGE 79
#define PW_MESSAGE_AUTHENTICATOR 80
#define PW_ARAP_CHALLENGE_RESPONSE 84
#define PW_ACCT_INTERIM_INTERVAL 85
#define PW_NAS_PORT_ID 87
#define PW_FRAMED_POOL 88
