/* AUTO-GENERATED HEADER FILE.  DO NOT EDIT. */
#define PW_NAS_IPV6_ADDRESS 95
#define PW_FRAMED_INTERFACE_ID 96
#define PW_FRAMED_IPV6_PREFIX 97
#define PW_LOGIN_IPV6_HOST 98
#define PW_FRAMED_IPV6_ROUTE 99
#define PW_FRAMED_IPV6_POOL 100
