/* AUTO-GENERATED HEADER FILE.  DO NOT EDIT. */
#define PW_ALLOWED_CALLED_STATION_ID 174
#define PW_EAP_PEER_ID 175
#define PW_EAP_SERVER_ID 176
#define PW_MOBILITY_DOMAIN_ID 177
#define PW_PREAUTH_TIMEOUT 178
#define PW_NETWORK_ID_NAME 179
#define PW_EAPOL_ANNOUNCEMENT 180
#define PW_WLAN_HESSID 181
#define PW_WLAN_VENUE_INFO 182
#define PW_WLAN_VENUE_LANGUAGE 183
#define PW_WLAN_VENUE_NAME 184
#define PW_WLAN_REASON_CODE 185
#define PW_WLAN_PAIRWISE_CIPHER 186
#define PW_WLAN_GROUP_CIPHER 187
#define PW_WLAN_AKM_SUITE 188
#define PW_WLAN_GROUP_MGMT_CIPHER 189
#define PW_WLAN_RF_BAND 190
